"use strict";
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getErrorMessageString = exports.getErrorMessage = exports.toErrorWithMessage = exports.isErrorWithMessage = void 0;
/* istanbul ignore next */
const isErrorWithMessage = (error) => {
    return (typeof error === 'object' &&
        error !== null &&
        'message' in error &&
        typeof error.message === 'string');
};
exports.isErrorWithMessage = isErrorWithMessage;
/* istanbul ignore next */
const toErrorWithMessage = (maybeError) => {
    if ((0, exports.isErrorWithMessage)(maybeError))
        return maybeError;
    try {
        return new Error(JSON.stringify(maybeError));
    }
    catch (_a) {
        // fallback in case there's an error stringifying the maybeError
        // like with circular references for example.
        return new Error(String(maybeError));
    }
};
exports.toErrorWithMessage = toErrorWithMessage;
const getErrorMessage = (error) => {
    return (0, exports.toErrorWithMessage)(error).message;
};
exports.getErrorMessage = getErrorMessage;
exports.getErrorMessageString = `export const isErrorWithMessage = (error: unknown): error is ErrorWithMessage => {
  return (
    typeof error === 'object' &&
    error !== null &&
    'message' in error &&
    typeof (error as Record<string, unknown>).message === 'string'
  );
};
export const toErrorWithMessage = (maybeError: unknown): ErrorWithMessage => {
  if (isErrorWithMessage(maybeError)) return maybeError;
  try {
    return new Error(JSON.stringify(maybeError));
  } catch {
    return new Error(String(maybeError));
  }
};
export const getErrorMessage = (error: unknown): string => {
  return toErrorWithMessage(error).message;
};`;
//# sourceMappingURL=get-error-message.js.map